#' Extract final URL & content type from response column
#'
#' Intended to be used in a [by()] or [dplyr::group_by()]
#' context, passing in the actual column
#'
#' @param a `resoonse` column from a call to [inspect()]
#' @export
#' @examples \dontrun{
#' library(dplyr)
#' library(tidyr)
#'
#' site <- inspect("http://rstudio.org")
#'
#' group_by(site, requestId) %>%
#'   summarise(orig_url = request$url[1], r = extract_url_and_content_type(response)) %>%
#'   unnest() %>%
#'   mutate(apex = apex_or_public_suffix(url))
#' }
extract_url_and_content_type <- function(response) {
  list(response[nrow(response), c("url", "mimeType")])
}

#' Infix NA tester and alt value chooser
#'
#' @param a,b values
#' @export
`%isna%` <- function(a, b) if (is.na(a)) b else a

#' Extract apex (or public suffix if no proper apex) from a FQDN or URL
#'
#' @param dom_or_url a character vector of domains or URLs
#' @export
#' @examples \dontrun{
#' library(dplyr)
#' library(tidyr)
#'
#' site <- inspect("http://rstudio.org")
#'
#' group_by(site, requestId) %>%
#'   summarise(orig_url = request$url[1], r = extract_url_and_content_type(response)) %>%
#'   unnest() %>%
#'   mutate(apex = apex_or_public_suffix(url))
#' }
apex_or_public_suffix <- function(dom_or_url) {
  vapply(dom_or_url, function(.x) {
    d <- curlparse::domain(.x)
    psl::apex_domain(d) %isna% psl::public_suffix(d)
  }, character(1), USE.NAMES = FALSE)
}

#' Test if Chrome Remote Interface is running on the given host/port
#'
#' @param cri_host,cri_port host and port Chrome remote interface is running on.
#' @export
cri_active <- function(cri_host = "localhost", cri_port = 9222L) {
  res <- httr::GET(url = "http://localhost:9222/")
  any(grepl("devtools", httr::content(res, as = "text", encoding = "UTF-8")))
}